const storage = window.localStorage

let sensors = []

let socket = undefined
let deviceUUID = undefined

let url = "http://prod.ctrlplus.team:3001"

let modalPortSettings = null;
let sensorSettings = null;
let settingsModal = null;
let archiveModal = null;

let lat = 0
let lon = 0
let speed = 0

const baudRates = [9600, 19200, 38400, 57600, 115200, 256000, 500000, 960000]
const intervals = {
    1: { seconds: 10, ms: 10000, value: '10 секунд' },
    2: { seconds: 30, ms: 30000, value: '30 секунд' },
    3: { seconds: 60, ms: 60000, value: '1 минута' },
    4: { seconds: 300, ms: 300000, value: '5 минут' },
    5: { seconds: 1800, ms: 1800000, value: '30 минут' },
    6: { seconds: 3600, ms: 3600000, value: '1 час' },
}

let usernameInput = undefined
let passwordInput = undefined
let loginButton = undefined

document.addEventListener('deviceready', onDeviceReady, false)

function onPositionSuccess(position) {
    lat = position.coords.latitude
    lon = position.coords.longitude
    speed = (position.coords.speed).toFixed(2) || 0
    // console.log(`POSITION CHANGED: Lat: ${position.coords.latitude}, lon: ${position.coords.longitude}, speed: ${position.coords.speed}, timestamp: ${position.timestamp}`)
};

function onPositionError(error) {
    showNotify('Ошибка местоположения: ' + error.code + '\n' + 'причина: ' + error.message + '\n')
}

function updateServerConnectionLabel(isConnOn) {
    const elements = document.getElementsByClassName('server-conn')
    for (const el of elements) {
        if (isConnOn) {
            el.innerText = "Связь с сервером установлена"
            el.classList.add('on')
        } else {
            el.innerText = "Связь с сервером отсутствует"
            el.classList.remove('on')
        }
    }
}

/**
 * Окраска элемента на главном экране и на экране авторизации в зависимости от статуса порта
 */
function updatePortStatus() {
    for (let dev in devStruct) {
        const device = devStruct[dev]
        const port = device.port + 1
        const elements = document.getElementsByClassName(`port-btn-${port}`)
        for (const el of elements) {
            if (device.statusOpened) {
                el.classList.add("connected")
            } else {
                el.classList.remove("connected")
            }
        }
    }
}

function onDeviceReady() {
    // Cordova is now initialized. Have fun!
    // Are you having fun yet?
    // No. You?
    //checkRoot();
    console.log('Running cordova-' + cordova.platformId + '@' + cordova.version);

    navigator.geolocation.watchPosition(onPositionSuccess, onPositionError)

    deviceUUID = device.uuid
    document.getElementById("uuid").innerText = `UUID: ${deviceUUID}`

    socket = io("http://prod.ctrlplus.team:3001", { transports: ["websocket"], reconnection: true, query: { "uuid": deviceUUID } });

    socket.on('connect', () => {
        console.log('Connected to the server');
        updateServerConnectionLabel(true)
        showNotify('Подключено к серверу')
        checkFileSize(cordova.file.dataDirectory + "packages.txt", function(size) {
            if (size > 0) {
                readData()
            } else if (size === 0) {
                console.log("File is empty, no data to send.")
            } else {
                console.log("Error occurred while checking file size.")
            }
        });
        // TODO записать адрес в storage
    });

    socket.on('disconnect', () => {
        showNotify('Отключено от сервера')
        updateServerConnectionLabel(false)
    });

    socket.on('error', () => {
        showNotify('Ошибка связи с сервером')
    })

    socket.on('connect_error', (e) => {
        showNotify('Не удалось подключиться к серверу')
    })

    socket.on('close', () => {
        showNotify('Соединение с сервером закрыто')
    })

    socket.on('reconnect', () => {
        showNotify('Переподключено к серверу')
    })

    socket.on('createPacketsSuccsses', (reply)=> {
        console.log("Покет отправлен успешно")
    });

    // Много модалок потому что очень классный был дизайн в фигме ставлю лайк мне очень понравилось все переделывать
    modalPortSettings = bootstrap.Modal.getOrCreateInstance(document.getElementById('portSettings'), { backdrop: 'static' });
    sensorSettings = bootstrap.Modal.getOrCreateInstance(document.getElementById('sensorSettings'), { backdrop: 'static' });
    settingsModal = bootstrap.Modal.getOrCreateInstance(document.getElementById('kudtSettings'), { backdrop: 'static' });
    archiveModal = bootstrap.Modal.getOrCreateInstance(document.getElementById('archiveSettings'), { backdrop: 'static' });

    usernameInput = document.getElementById("login-input")
    passwordInput = document.getElementById("password-input")
    loginButton = document.getElementById("login-btn")

    usernameInput.addEventListener("input", checkInputs)
    passwordInput.addEventListener("input", checkInputs)

    if (storage.getItem("sensors")) {
        // storage.setItem("sensors", "")

        sensors = JSON.parse(storage.getItem("sensors"))
        console.log("SENSORS:", sensors)

        const sensorsCount1 = sensors.filter(item => item.comPort == 1).length
        document.getElementById('sensors-count-1').innerText = 'Кол-во датчиков: ' + sensorsCount1
        const sensorsCount2 = sensors.filter(item => item.comPort == 2).length
        document.getElementById('sensors-count-2').innerText = 'Кол-во датчиков: ' + sensorsCount2

        sensors.forEach(sensor => {
            createSensorElement(sensor)
        })
    }

    if (storage.getItem("url")) {
        url = storage.getItem("url")
    }
    console.log("URL:", url)

    if (storage.getItem("comports")) {
        console.log("PORTS LOADING...")
        let tmpDevStruct = JSON.parse(storage.getItem("comports"))
        console.log(tmpDevStruct)

        if (typeof tmpDevStruct == 'object') {

            tmpDevStruct[0].settingsChanged = false
            tmpDevStruct[0].statusOpened = false
            tmpDevStruct[1].settingsChanged = false
            tmpDevStruct[1].statusOpened = false

            console.log("PORTS LOADED")
            devStruct = { ...tmpDevStruct }
            devStruct[0].busy = false
            devStruct[1].busy = false
        }
        tmpDevStruct = ''
        updatePortLabels()
        console.log("devStruct", devStruct)
    }

    var inputFields = document.querySelectorAll('.user-input');

    inputFields.forEach(function (inputField) {
        inputField.addEventListener('keyup', function (event) {
            if (event.key === "Enter") {
                inputField.blur()
            }
        })
    })
}

function updateStorage() {
    storage.setItem("sensors", JSON.stringify(sensors))
    storage.setItem("comports", JSON.stringify(devStruct))
}

/** Добавление на экран плашки с инфой о датчике */
function createSensorElement(sensor) {
    const comPort = sensor.comPort
    const container = document.getElementById(`sensors-port-${comPort}`)
    sensor.prevExec = Date.now()
    sensor.isRunning = false

    const sensorItem = document.createElement('button')
    sensorItem.classList.add('bt-n')
    sensorItem.classList.add('mb-1')
    sensorItem.setAttribute("id", `sensor-${sensor.id}`)
    sensorItem.setAttribute("com-port", `${comPort}`) // FIXME вроде это теперь нафиг не надо, только не сломать прогресс бар
    sensorItem.setAttribute("sensor-id", `${sensor.id}`) // FIXME вроде это теперь нафиг не надо, только не сломать прогресс бар
    sensorItem.setAttribute("onclick", `updateOrDeleteSensor(${sensor.id})`)
    const sensorType = sensorsAPI[sensor.type].name
    const interval = intervals[sensor.frequency].value

    sensorItem.innerHTML = `
        <div class="info-btn">
            <div class="d-flex">
                <label class="select-label">СИ №${sensor.id}</label>
                <label class="select-value d-flex align-items-center">
                    (${sensorType}, ${sensor.address}, ${interval})
                </label>
            </div>
        </div>
        
        <div class="progress" role="progressbar"  style="height: 3px">
            <div class="progress-bar bg-success" style="width: 0%"></div>
        </div>`
    container.appendChild(sensorItem)
}

// Проверка, что логин и пароль заполнены, чтобы кнока "Войти стала активной"
function checkInputs() {
    const usernameValue = usernameInput.value
    const passwordValue = passwordInput.value

    if (usernameValue !== "" && passwordValue !== "") {
        loginButton.removeAttribute("disabled")
    } else {
        loginButton.setAttribute("disabled", true)
    }
}

function login() {
    // FIXME только для отладки
    document.getElementById('auth').style.display = 'none'
    document.getElementById('main').style.display = ''

    // // TODO вообще-то шифровать пароль тут хорошо бы)))
    // const credentials = { login: usernameInput.value, password: passwordInput.value }
    // console.log("Trying to log in with credentials:", credentials)

    // if (socket) {
    //     socket.emit('authenticate', credentials)
    //     socket.on('authReply', (payload) => {
    //         if (payload.status == "ok") {
    //             document.getElementById('auth').style.display = 'none'
    //             document.getElementById('main').style.display = ''
    //             const authWarning = document.getElementById('auth-warning')
    //             authWarning.style.display = 'none'
    //             // TODO хранить токеновое
    //             // TODO сделать таймаут на ожидание authReply
    //         } else {
    //             const authWarning = document.getElementById('auth-warning')
    //             authWarning.style.display = ''
    //             usernameInput.value = ''
    //             passwordInput.value = ''
    //         }
    //     })
    // } else {
    //     console.log("OOPS! Socket is undefined!")
    // }
}
function logout() {
    document.getElementById('auth').style.display = ''
    document.getElementById('main').style.display = 'none'
}

/** Переход на экран "Подлючение датчиков" с экрана "Настойки" */
function goBack() {
    document.getElementById('settings').style.display = 'none'
    document.getElementById('main').style.display = ''
}

/** Переход на экран "Настройки" с экрана "Подключение датчиков" */
function openSettings() {
    document.getElementById('settings').style.display = ''
    document.getElementById('main').style.display = 'none'

    const siOptions = document.getElementById('availableSi')
    // TODO заполнить селект опшнс
    // но есть ощущэние, что их надо заполнять не датчиками из sensors, а откуда-то изве
}

function setText(elementId, text) {
    const element = document.getElementById(elementId)
    element.innerText = text
}

function showSettingsModal() {
    settingsModal.show()
}

function showArchiveModal() {
    archiveModal.show()
}