var logcat = '';

let devStruct = {
    0: {
        port: 0,
        baudRate: 19200,
        data: undefined,
        error: undefined,
        settingsChanged: false,
        statusOpened: false,
        dataBits: 8,
        stopBits: 1,
        parity: 1,
        busy: false
    },
    1: {
        port: 1,
        baudRate: 19200,
        data: undefined,
        error: undefined,
        settingsChanged: false,
        statusOpened: false,
        dataBits: 8,
        stopBits: 1,
        parity: 1,
        busy: false
    },
}
let currPort = 0;
let currPrib = -1;
let currKey = -1;
let currField = [];
let replyData = null

document.addEventListener('deviceready', onModbusReady, false);

function onModbusReady() {
    logcat = document.getElementById("logcat");
    modbus_exec();
}

function writeData(dev, data) {
    serial.write(dev, data, () => { successWriteCallback(dev) }, () => { errorWriteCallback(dev) });
}
function writeDataHex(dev, data) {
    currField = hexToUint8Array(data);
    serial.writeHex(dev, data, () => { successWriteCallback(dev) }, () => { errorWriteCallback(dev) });
}

function arrayBufferToString(buffer) {
    let decoder = new TextDecoder("utf-8");
    return decoder.decode(buffer);
}
// function arrayBufferToHex(buffer) {
//     let byteArray = new Uint8Array(buffer);
//     let byteTrim = removeSubArray(byteArray, currField);
//     console.log("BYTE ARRAY:", byteArray)
//     console.log("BYTE TRIM:", byteTrim)
//     console.log("CURR FIELD:", currField)
//     let hexParts = [];
//     for (let byte of byteTrim) {
//         let hexByte = byte.toString(16);
//         hexParts.push(hexByte.padStart(2, '0'));  // pad with '0' if byte is less than 0x10
//     }
//     const result = hexParts.join('')
//     console.log("RESULT:", result)
//     return result;
// }

function bufferToHex(byteTrim) {
    let hexParts = [];
    for (let byte of byteTrim) {
        let hexByte = byte.toString(16);
        hexParts.push(hexByte.padStart(2, '0'));  // pad with '0' if byte is less than 0x10
    }
    const result = hexParts.join('')
    console.log("RESULT:", result)
    return result;
}

function hexToUint8Array(hexString) {
    if (hexString.length % 2 !== 0) {
        throw new Error('Invalid hex string length');
    }

    const arrayBuffer = new Uint8Array(hexString.length / 2);

    for (let i = 0; i < hexString.length; i += 2) {
        const byteValue = parseInt(hexString.substr(i, 2), 16);
        if (isNaN(byteValue)) {
            throw new Error('Invalid hex string');
        }
        arrayBuffer[i / 2] = byteValue;
    }

    return arrayBuffer;
}
function isSubArray(main, sub) {
    for (let i = 0; i <= main.length - sub.length; i++) {
        let matched = true;
        for (let j = 0; j < sub.length; j++) {
            if (main[i + j] !== sub[j]) {
                matched = false;
                break;
            }
        }
        if (matched) {
            return i;  // Return the starting index of the subarray in the main array
        }
    }
    return -1;  // Subarray not found
}

function removeSubArray(main, sub) {
    let index;
    while ((index = isSubArray(main, sub)) !== -1) {
        let before = main.slice(0, index);
        let after = main.slice(index + sub.length);
        main = new Uint8Array([...before, ...after]);
    }
    return main;
}
function formatTime(date) {
    if (date == undefined) date = new Date();
    const pad = (num) => num < 10 ? '0' + num : num;

    let hours = pad(date.getHours());
    let minutes = pad(date.getMinutes());
    let seconds = pad(date.getSeconds());

    return `${hours}:${minutes}:${seconds}`;
}
function twoBytesToFloat(dataBuff, position) {
    // Extract the two bytes from the data
    let data = new Uint8Array(dataBuff);
    let byte1 = data[position];
    let byte2 = data[position + 1];

    // Combine the bytes. This is just an example. The real combination logic 
    // will depend on how the float is represented in your system.
    let combined = (byte1 << 8) | byte2;

    // Convert combined value to float. Again, this is just an example.
    // The exact conversion will depend on your system's specifics.
    let floatValue = combined / 100.0; // example: treat as fixed-point with two decimal places

    return floatValue;
}
function readParser(data, currKey, type) {
    // ОПИСАНИЕ ОТВЕТА
    // 0 байт 1-255 0х01-0хFF АДРЕС УСТРОЙСТВА
    // 1 байт код функции
    // 2 байт сколько байтов в ответе
    // 3...n полезная часть ответа
    // n + 1 CRC16 1
    // n + 2 CRC16 2
    console.log("DATA:", data)
    console.log("CURR KEY:", currKey)
    console.log("TYPE:", type)

    const byteArray = new Uint8Array(data);

    // ненужновое
    const parsed = bufferToHex(byteArray);

    let value = parsed
    // twoBytesToFloat(byteArray, 3);

    const meaningfulData = byteArray.subarray(1, byteArray.length - 2);
    console.log("meaningful data: ", meaningfulData);
    const bytesCount = meaningfulData.length;

    const buffer = new ArrayBuffer(meaningfulData.length); // Use meaningfulData.length
    const dataView = new DataView(buffer);

    meaningfulData.forEach((value, index) => {
        dataView.setUint8(index, value);
    });

    value = bufferToHex(meaningfulData);

    const lastTwoBytes = (byteArray[byteArray.length - 1] << 8) | byteArray[byteArray.length - 2]
    const smth = byteArray.subarray(0, byteArray.length - 2);
    const crc = crc16(smth);

    const packet = sensorsAPI[type].fields[currKey]
    const name = packet.sys_name;

    if (crc != lastTwoBytes) {
        console.log(`CRC doesn't match for ${name}: ${crc} - ${lastTwoBytes}`)
        //return
    } else {
        console.log(`CRC matches for ${name}!!!. Parsed=${parsed}. CRC: ${crc}, SMTH: ${smth}`)
    }


    //Вы абасралис и забыли задефайнить переменную packet
    const totalSize = dataView.byteLength
    let dataTypeSize = 4
    let offset = 0
    if (totalSize != 0) {
        switch (packet.type) {
            case "float":
                if (bytesCount >= 4) {
                    dataTypeSize = 4
                    offset = totalSize - dataTypeSize
                    value = dataView.getFloat32(offset)
                } else {
                    dataTypeSize = 2
                    offset = totalSize - dataTypeSize
                    value = dataView.getInt16(offset)
                }
                break
            case "int32":
                if (bytesCount >= 4) {
                    dataTypeSize = 4
                    offset = totalSize - dataTypeSize
                    value = dataView.getInt32(offset)
                } else {
                    dataTypeSize = 2
                    offset = totalSize - dataTypeSize
                    value = dataView.getInt16(offset)
                }
                break
            case "int16":
            case "int": // TODO можно везде одинаково назвац
                dataTypeSize = 2
                offset = totalSize - dataTypeSize
                value = dataView.getInt16(offset)
                break
            case "uint16":
            case "uint8": // в игле было написано, что uint8 - то же что uint16, первый байт всегда 0
                dataTypeSize = 2
                offset = totalSize - dataTypeSize
                value = dataView.getUint16(offset)
                break
            case "string":
            case "str16":
            case "str32":
                const decoder = new TextDecoder("utf-8")
                value = decoder.decode(dataView)
                break
            default:
                console.log("Вы абасралис и забыли указать какой-то тип тут")

        }
    }


    if (parsed.length > 0) {
        printLogcat('<small class="muted">' + formatTime() + '</small>&nbsp;' + parsed + ' |  ' + value);
        //console.log(parsed);
        devStruct[0].data = data;
    }

    return value
}


function modbus_exec() {
    setInterval(modbus_queue, 1000);
}
let queue = [];
function modbus_queue() {
    for (const device of sensors) {
        if (device.prevExec == undefined) device.prevExec = Date.now();
        if (device.isRunning == undefined) device.isRunning = false;
        if (device.priority == undefined) device.priority = 0;
        if (Date.now() - device.prevExec >= intervals[Number(device.frequency)].ms && !device.isRunning) {
            if (!queue.includes(device.id)) {
                queue.push(device.id)
                console.log("Device added to queue:", device.id)
            }
        }
        /*   currPrib = pribIdx;
           currPort = device.comPort - 1

           if (devStruct[currPort].busy) {
               console.log(`COM PORT ${device.comPort} IS BUSY, SKIPPING SENSOR ${device.id}`)
               return
           } else {
               devStruct[0].busy = true
               devStruct[1].busy = true

               device.isRunning = true;
               console.log("NEW CYCLE--------------------------------");
               if (devStruct[currPort].statusOpened) {
                   console.log("Прибор: " + sensorsAPI[device.type].name + " " + device.type + " " + intervals[Number(device.frequency)].ms);

                   if (device.type != "4") {
                       processEntries(device);
                   }
                   else {
                       console.log("Прибор не поддерживается");
                       devStruct[0].busy = false
                       devStruct[1].busy = false
                       device.isRunning = false
                   }
               }
               else {
                   console.log("Порт закрыт");
                   device.isRunning = false;
                   device.prevExec = Date.now();
               }
           }
       }*/
        // some routine to update progress bar while we waiting for next cycle
        const progress = Date.now() - device.prevExec;
        const roughPercentage = Math.round(progress / intervals[Number(device.frequency)].ms * 100);
        let percentage = (roughPercentage > 100) ? 100 : roughPercentage;
        percentage = (percentage < 0) ? 0 : percentage;
        percentage = device.isRunning ? 100 : percentage;
        // FIXME по-моему тут надо попрвить sensor-id, но это не точно
        const progressBar = document.querySelector("button[sensor-id='" + device.id + "'] .progress-bar")
        progressBar.setAttribute("style", "width: " + percentage + "%");
        if (device.isRunning) {
            progressBar.classList.add("progress-bar-animated")
            progressBar.classList.add("progress-bar-striped")
        }
        else {
            progressBar.classList.remove("progress-bar-animated")
            progressBar.classList.remove("progress-bar-striped")
        }
    }

    if (queue.length > 0 && !devStruct[0].busy && !devStruct[1].busy && devStruct[0].statusOpened && devStruct[1].statusOpened) {
        devStruct[0].busy = true
        devStruct[1].busy = true
        const device_id = queue.shift() - 1
        const device = sensors[device_id]
        device.isRunning = true
        console.log("Processing queued device:", sensorsAPI[device.type].name)
        processEntries(device)
    }

    for (let kPort in devStruct) {
        if (devStruct[kPort].settingsChanged) {
            updateSerialPort(kPort)
            updatePortLabels()
        }
    }
}

// Новая функция для обработки всех полей
async function processEntries(device) {
    const entries = Object.entries(sensorsAPI[device.type].fields);
    // TODO узнать у Костика зачем delayAccumulation и delay_mult, они вроде не используются...
    let delayAccumulation = 0;
    let startTime = Date.now();

    const packages = {}

    processPacketFields(packages, device, Object.entries(sensorsAPI[device.type].packetFields))

    for (let [key, value] of entries) {
        if (value.en) {
            // FIXME array
            let currentPackage = packages[value.package_num[0]]
            if (!currentPackage) {
                packages[value.package_num[0]] = { packetNum: value.package_num[0], uuid: deviceUUID }
                currentPackage = packages[value.package_num[0]]
            }
            console.log("KEY: ", key, "NAME: ", value.name);
            let allParsedData = [];
            let delay_mult = (Array.isArray(value.offset)) ? value.offset.length * 2 : 1;

            if (Array.isArray(value.offset)) {
                for (let subOffset of value.offset) {
                    const parsedData = await queryModbus(device, value, key, subOffset, currentPackage);
                    console.log("parsedData:", parsedData)
                    allParsedData.push(parsedData);
                }
            } else {
                const parsedData = await queryModbus(device, value, key, undefined, currentPackage);
                allParsedData.push(parsedData);
            }

            // console.log("All parsed data:", allParsedData)

            let resultValue = undefined

            // If allParsedData has more than one item, combine them
            if (allParsedData.length > 1) {
                const combinedParsedData = combineValues(allParsedData);
                console.log("COMBINED PARSED DATA:", combinedParsedData);
                // Use combinedParsedData as needed...
                resultValue = combinedParsedData
            } else {
                // Use allParsedData[0] directly as needed...
                resultValue = allParsedData[0]
            }

            // currentPackage.data[value.sys_name] = resultValue
            // currentPackage.timeCreate = Date.now()

            delayAccumulation += 520 * delay_mult;
            //console.log("DELAY: ", delayAccumulation);
        }
    }

    for (let pkg in packages) {
        packages[pkg].uuid = deviceUUID
    }
    const packagesArr = Object.values(packages)
    // TODO store if connection is lost, send on reconnection

    if (socket.connected) {
        console.log("Sending data to server:", packagesArr)
        socket.emit('createPackets', packagesArr)
    } else {
        console.log("OOPS! Not connected")
        saveData(packagesArr)
    }

    device.prevExec = Date.now();
    device.isRunning = false;
    devStruct[0].busy = false;
    devStruct[1].busy = false;
    console.log("Время выполнения: ", Date.now() - startTime);
}

// Запрос данных
async function queryModbus(pribor, value, cmdIdx, subOffset, obj) {
    const startRegister = (subOffset !== undefined) ? value.start + subOffset : value.start;
    const startAddress = pribor.address;
    const finalOffset = (subOffset !== undefined) ? subOffset : value.offset;
    const req = createModbusRequest(startAddress, value.coil, value.start, finalOffset, value.size);
    printLogcat("PORT " + pribor.comPort + ": " + req);
    sendBinary(currPort, req);
    replyData = null;

    const waitForData = new Promise((resolve) => {
        const checkData = () => {
            if (replyData !== null) {
                const data = replyData;
                const parsed = readParser(data, cmdIdx, pribor.type);
                obj[value.sys_name] = parsed;
                const bArr = new Uint8Array(data);
                obj[value.sys_name + "_hex"] = bufferToHex(bArr);
                obj.pack_dt = Math.floor(Date.now() / 1000)
                resolve(parsed);
            } else {
                setTimeout(checkData, 50); // Check every 50 ms
            }
        };
        checkData();
    });

    const timeout = new Promise((resolve) => {
        setTimeout(() => resolve(false), 500); // CHANGE BACK TO 2000 !!!
        obj[value.sys_name] = "";
        obj[value.sys_name + "_hex"] = "";
    });

    const parsedData = await Promise.race([waitForData, timeout]);
    return parsedData;
}

function combineValues(dataArray) {
    // Assuming data values need to be joined as strings
    return dataArray.join('');
}

function printLogcat(html) {
    logcat.insertAdjacentHTML("afterbegin", '<div class="list-group-item">' + html + '</div>');
    if (logcat.children.length > 10) {
        logcat.removeChild(logcat.children[(logcat.children.length - 1)]);
    }
}
function createModbusRequest(slaveAddress, coilAddress, startAddress, offsetAddress, numRegisters) {
    const buffer = new Uint8Array(6)
    buffer[0] = slaveAddress
    buffer[1] = coilAddress

    const registerAddress = startAddress + offsetAddress
    console.log("START ADDRESS:", startAddress, "OFFSET ADDRESS:", offsetAddress)
    console.log("REGISTER ADDRESS:", registerAddress)
    buffer[2] = registerAddress >> 8
    buffer[3] = registerAddress & 0xff
    console.log("BUFFER 2:", buffer[2], "BUFFER 3:", buffer[3])


    const numRegistersUint8_1 = numRegisters & 0xff
    const numRegistersUint8_2 = numRegisters >> 8

    buffer[4] = numRegistersUint8_2
    buffer[5] = numRegistersUint8_1

    const computedCrc = crc16(buffer)
    const computedCrcUint8_1 = computedCrc & 0xff
    const computedCrcUint8_2 = computedCrc >> 8

    const crcBuffer = new Uint8Array(2)

    crcBuffer[0] = computedCrcUint8_1
    crcBuffer[1] = computedCrcUint8_2

    const mergedArray = new Uint8Array(8)
    mergedArray.set(buffer)
    mergedArray.set(crcBuffer, 6)


    return mergedArray//bytesToHex(mergedArray)
}

// Накопипастила чево-то из интернетового, потому что мы не в ноде(
function crc16(buffer) {
    var crc = 0xFFFF;
    var odd;

    for (var i = 0; i < buffer.length; i++) {
        crc = crc ^ buffer[i];

        for (var j = 0; j < 8; j++) {
            odd = crc & 0x0001;
            crc = crc >> 1;
            if (odd) {
                crc = crc ^ 0xA001;
            }
        }
    }

    return crc;
};

function bytesToHex(uint8Array) {
    return Array.from(uint8Array).map(byte => {
        return ('0' + byte.toString(16)).slice(-2);  // Ensure each byte is 2 characters long
    }).join('');
}

function processPacketFields(packages, device, packetFields) {
    for (let [key, value] of packetFields) {
        for (let packNum of value.package_num) {
            let currentPackage = packages[packNum]
            if (!currentPackage) {
                // packetNum = DataType
                packages[packNum] = { packetNum: packNum }
                currentPackage = packages[packNum]
            }

            let data = undefined
            switch (value.sys_name) {
                case 'data_type':
                    data = packNum
                    break
                case 'unix_time':
                case 'pack_dt': // ???
                case 'timestamp': // FIXME?
                    data = Math.floor(Date.now() / 1000)
                    break
                case 'latitude':
                case 'pack_lat':
                    data = lat
                    break
                case 'longitude':
                case 'pack_lon':
                    data = lon
                    break
                // zochem...
                case 'geo':
                    data = `${lat}, ${lon}`
                    break
                case 'speed':
                    data = speed
                    break;
                // напряжение бортовой сети))))))))))))))))))))
                case 'voltage':
                    data = 0
                    break
                case 'port_id':
                    data = device.comPort
                    break
                case 'source_address':
                    data = device.address
                    break;
                // id, imei, arhiv_date, voltage, dt???
            }

            currentPackage[value.sys_name] = data
        }
    }
}
