/** Для запоминания айдишника сенсора на который тапнули (для обновления или удаления) */
let selectedSensorId = undefined

/**
 * Открытие меню для выбора COM-порта
 * @param {number} id номер COM-порта (1/2) 
 */
function togglePortOptions(id) {
    const idx = id == 1 ? 0 : 1

    document.getElementById("modalPortNum").innerHTML = id

    const selectBaud = document.getElementById("editPortBauds")
    selectBaud.value = devStruct[idx].baudRate
    const selectDataBits = document.getElementById("editPortDataBits")
    selectDataBits.value = devStruct[idx].dataBits
    const selectStopBits = document.getElementById("editPortStopBits")
    selectStopBits.value = devStruct[idx].stopBits
    const selectParity = document.getElementById("editPortParity")
    selectParity.value = devStruct[idx].parity

    const saveBtn = document.getElementById("modalSavePort")
    saveBtn.setAttribute("com-port", idx)

    modalPortSettings.show()
}

/**
 * COM-порт
 * @param {number} id номер COM-порта (1/2)
 * @param {Object} item выбранное значение
 */
function updatePort(id, item) {
    document.getElementById(`port-btn-${id}`).classList.toggle('open')
    document.getElementById(`port-btn-${id}`).classList.add('selected')
    document.getElementById(`selected-port-${id}`).innerText = item.value
    document.getElementById(`add-btn-${id}`).disabled = false
    document.getElementById(`port-options-${id}`).style.display = 'none'
    const portID = id == 1 ? 0 : 1
    devStruct[portID] = item.value
    devStruct[portID].settingsChanged = true
}

/** Сброс выбранных значений в модалке с настройками датчика */
function dropSettingsSelection() {
    document.getElementById("cp").disabled = false
    document.getElementById("cp").selectedIndex = 0
    document.getElementById("siType").selectedIndex = 0
    document.getElementById("pollingRate").selectedIndex = 0
    document.getElementById("siAddress").value = ""
    selectedSensorId = undefined
}

function updateOrDeleteSensor(sensorId) {
    console.log("UPDATE OR DELETE SENSOR ", sensorId)
    selectedSensorId = sensorId
    const selectedSensor = sensors.find(item => item.id == sensorId)
    console.log(selectedSensor)
    document.getElementById("cp").selectedIndex = selectedSensor.comPort
    document.getElementById("siType").selectedIndex = selectedSensor.type
    document.getElementById("pollingRate").selectedIndex = selectedSensor.frequency
    document.getElementById("siAddress").value = selectedSensor.address
    showSensorSettings(false, "Изменение СИ №" + sensorId)
}

/** Открытие модалки для добавления датчика или изменения/обновления */
function showSensorSettings(isNew, title) {
    if (isNew) {
        document.getElementById("delete-btn").style.display = "none"
        document.getElementById("sensorSettingsLabel").innerText = `Добавление СИ №${sensors.length == 0 ? 1 : 1 + sensors[sensors.length - 1].id}`
    } else {
        document.getElementById("cp").disabled = true
        document.getElementById("delete-btn").style.display = ""
        document.getElementById("sensorSettingsLabel").innerText = title
    }
    sensorSettings.show()
}

function deleteElementById(elementId) {
    const itemToDelete = document.getElementById(elementId)
    itemToDelete.parentNode.removeChild(itemToDelete)
}

function portSave() {
    const saveBtn = document.getElementById("modalSavePort")
    const idx = saveBtn.getAttribute("com-port")
    const bauds = document.querySelector("#editPortBauds").value
    const dataBits = document.querySelector("#editPortDataBits").value
    const stopBits = document.querySelector("#editPortStopBits").value
    const parity = document.querySelector("#editPortParity").value
    console.log("idx:", idx, "bauds:", bauds, "dataBits:", dataBits, "stopBits:", stopBits, "parity:", parity)
    devStruct[idx].baudRate = bauds
    devStruct[idx].dataBits = dataBits
    devStruct[idx].stopBits = stopBits
    devStruct[idx].parity = parity
    devStruct[idx].settingsChanged = true
    console.log("devStruct:", devStruct)
    updateStorage()
    updatePortLabels()
    modalPortSettings.hide()
}

function updatePortLabels() {
    const portBtns1 = document.getElementsByClassName('port-btn-1')
    for (const portBtn of portBtns1) {
        portBtn.classList.add('selected')
    }

    const portLabels1 = document.getElementsByClassName('cp1-lbl')
    for (const portLbl of portLabels1) {
        portLbl.innerText = `(${devStruct[0].baudRate}, ${devStruct[0].dataBits}, ${devStruct[0].stopBits}, ${devStruct[0].parity})`
    }

    const portBtns2 = document.getElementsByClassName('port-btn-2')
    for (const portBtn of portBtns2) {
        portBtn.classList.add('selected')
    }
    const portLabels2 = document.getElementsByClassName('cp2-lbl')
    for (const portLbl of portLabels2) {
        portLbl.innerText = `(${devStruct[1].baudRate}, ${devStruct[1].dataBits}, ${devStruct[1].stopBits}, ${devStruct[1].parity})`
    }
}

function saveSensor() {
    const comPort = document.getElementById("cp").value
    if (comPort == 0) {
        alert("Выберите COM-порт")
        return
    }

    const type = document.getElementById("siType").value
    if (type == 0) {
        alert("Выберите тип датчика")
        return
    }
    const address = parseInt(document.getElementById("siAddress").value)
    if (!address || address < 1 || address > 255) {
        alert("Указан некорректный адрес")
        return
    }
    const pollingRate = document.getElementById("pollingRate").value
    if (pollingRate == 0) {
        alert("Выберите частоту опроса")
        return
    }

    console.log(`Saving sensor: address: ${address}, type: ${type}, polling rate: ${pollingRate}`)

    if (selectedSensorId) {
        const confirmResult = confirm(`Вы действительно хотите обновить данные для СИ №${selectedSensorId}?`)
        if (!confirmResult) {
            return
        }
        const idx = sensors.findIndex(item => item.id == selectedSensorId)
        if (idx == -1) {
            alert("Не удалось обновить данные")
            return
        }
        const selectedSensor = sensors[idx]
        selectedSensor.type = type
        selectedSensor.address = address
        selectedSensor.frequency = pollingRate // TODO rename

        const elementToUpdate = document.querySelector(`#sensor-${selectedSensorId} .select-value`)
        if (!elementToUpdate) {
            alert("Не удалось обновить данные")
            return
        }
        elementToUpdate.innerText =  `(${sensorsAPI[type].name}, ${address}, ${intervals[pollingRate].value})`
        // TODO наверное надо как-то по-умному еще где-то что-то обновить
    } else {
        // добавление нового датчика
        const newSensor = {comPort: comPort}
        newSensor.id = sensors.length == 0 ? 1 : sensors[sensors.length - 1].id + 1
        const confirmResult = confirm(`Вы действительно хотите обновить добавить СИ №${newSensor.id}?`)
        if (!confirmResult) {
            return
        }
        newSensor.type = type
        newSensor.address = address
        newSensor.frequency = pollingRate // TODO rename

        createSensorElement(newSensor)
        sensors.push(newSensor)

        console.log("SENSORS:", sensors)
    }

    updateStorage()

    sensorSettings.hide()
    dropSettingsSelection()
}

function deleteSensor() {
    const deleteConfirmed = confirm(`Вы действительно хотите удалить СИ №${selectedSensorId}?`)
    if (deleteConfirmed) {
        sensorSettings.hide()
        deleteElementById(`sensor-${selectedSensorId}`)
        sensors = sensors.filter(item => item.id != selectedSensorId)
        updateStorage()
    }
    dropSettingsSelection()
}