//const filePath = cordova.file.dataDirectory + "packages.txt";
function writeToFile(fileEntry, dataObj, append = true) {
    fileEntry.createWriter(function (fileWriter) {
        fileWriter.onwriteend = function () {
            console.log("Successful file write...");
        };
        fileWriter.onerror = function (e) {
            console.log("Failed file write: " + e.toString());
        };
        if (append) {
            fileWriter.seek(fileWriter.length); // Go to the end of the file
        }
        var blob = new Blob([dataObj], { type: 'text/plain' });
        fileWriter.write(blob);
    });
}

function appendToFile(fileEntry, dataObj) {
    fileEntry.createWriter(function (fileWriter) {
        fileWriter.onwriteend = function () {
            console.log("Successful file append...");
        };
        fileWriter.onerror = function (e) {
            console.log("Failed file append: " + e.toString());
        };
        fileWriter.seek(fileWriter.length); // Go to the end of the file
        var blob = new Blob([JSON.stringify(dataObj) + '\n'], { type: 'text/plain' });
        fileWriter.write(blob);
    });
}


function saveData(data) {
    window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (directoryEntry) {
        directoryEntry.getFile("packages.txt", { create: true }, function (fileEntry) {
            appendToFile(fileEntry, JSON.stringify(data));
        });
    }, function (e) {
        console.log("Error accessing file system: " + e.toString());
    });
}

function clearData() {
    window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (directoryEntry) {
        directoryEntry.getFile("packages.txt", { create: false }, function (fileEntry) {
            // Delete the file
            fileEntry.remove(function () {
                console.log("File removed successfully.");

                // Create a new file after deletion
                directoryEntry.getFile("packages.txt", { create: true }, function (newFileEntry) {
                    console.log("New file created successfully.");
                }, errorHandler);

            }, errorHandler);
        }, errorHandler);
    }, errorHandler);
}

function errorHandler(error) {
    console.log("FileSystem Error: " + error.code);
}

function readFile(fileEntry) {
    fileEntry.file(function (file) {
        var reader = new FileReader();
        reader.onloadend = function () {
            var lines = this.result.split('\n');
            var linesLength = lines.length;
            lines.forEach(function (line) {
                if (line.trim()) {
                    var data = JSON.parse(line);
                    data = JSON.parse(data);
                    socket.emit('createPackets', data);

                }
                linesLength--;
                console.log(linesLength);
                if (linesLength == 0) {
                    clearData();
                }
            });
        };
        reader.readAsText(file);
    }, function (e) {
        console.log("Error reading file: " + e.toString());
    });
}
function readData() {
    window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (directoryEntry) {
        directoryEntry.getFile("packages.txt", { create: false }, function (fileEntry) {
            readFile(fileEntry);
            // Optionally delete the file after successful processing
        });
    }, function (e) {
        console.log("Error accessing file system: " + e.toString());
    });
}
function checkFileSize(filePath, callback) {
    window.resolveLocalFileSystemURL(filePath, function (fileEntry) {
        // File exists, now get its size
        fileEntry.file(function (file) {
            var fileSize = file.size; // Size in bytes
            callback(fileSize);
        }, function (error) {
            console.log("Error retrieving file: " + error.toString());
            callback(-1); // Indicate an error
        });
    }, function (error) {
        if (error.code === 1) { // FILE_NOT_FOUND_ERR
            // File doesn't exist, so create it
            console.log("File not found. Creating file...");
            createFile(filePath, function (newFileEntry) {
                if (newFileEntry) {
                    callback(0); // New file, size is 0
                } else {
                    callback(-1); // Error creating file
                }
            });
        } else {
            console.log("Error resolving file system URL: " + error.toString());
            callback(-1); // Indicate an error
        }
    });
}
function createFile(filePath, callback) {
    window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (directoryEntry) {
        directoryEntry.getFile("packages.txt", { create: true, exclusive: false }, function (fileEntry) {
            console.log("File created: " + fileEntry.fullPath);
            callback(fileEntry);
        }, function (error) {
            console.log("Error creating file: " + error.toString());
            callback(null);
        });
    }, function (error) {
        console.log("Error accessing file system: " + error.toString());
        callback(null);
    });
}