
var websocket = null
var reconnectInterval = 30000
var reconnectTimer = null
var lastConn = null

document.addEventListener('deviceready', socketInit, false)

function socketInit() {
    websocket = cordova.plugins.wsserver;
    websocket.stop()
    websocket.start(8081, {
        'onFailure' :  function(addr, port, reason) {
            console.log('Stopped listening on %s:%d. Reason: %s', addr, port, reason);
            setTimeout(socketInit, 20000);
        },
        'onOpen' : function(conn) {
            lastConn = conn
            console.log('A user connected from %s', conn.remoteAddr);
            setPorts(true)
        },
        'onMessage' : function(conn, msg) {
            console.log('Message from', conn.remoteAddr);
            if (msg instanceof ArrayBuffer) {
                replyData = new Uint8Array(msg);
                //console.log("Received binary data:", replyData);
            } else {
                console.log("Received text message:", msg);
            }
        },
        'onClose' : function(conn, code, reason, wasClean) {
            console.log('A user disconnected from %s', conn.remoteAddr);
            setPorts(false)
        },
        'onError' : function(conn, err) {
            console.log('An error occured with connection from %s', conn.remoteAddr);
            setPorts(false)
            console.log(err);
        },
        'tcpNoDelay' : true
    }, function onStart(addr, port) {
        console.log('Listening on %s:%d', addr, port);
    }, function onDidNotStart(reason) {
        console.log('Did not start. Reason: %s', reason);
    })
}

function updateSerialPort(port) {
    let dev = devStruct[port]
    if (dev.statusOpened && dev.settingsChanged) {
        const port = (dev.port == 0) ? "A" : "B"
        let portString = port + "," + dev.baudRate + "," + dev.dataBits + "," + dev.parity + "," + dev.stopBits
        websocket.send(lastConn, portString)
        dev.settingsChanged = false
    }
}
function sendBinary(port, data) {
    //check if data is bytearray
    if (data.constructor !== Uint8Array) {
        console.error("Data is not a Uint8Array")
        return
    }
    websocket.send(lastConn, (port == 0) ? "P,0" : "P,1")
    websocket.send(lastConn, data, { binary: true })
}

function setPorts(open) {
    devStruct[0].statusOpened = open
    devStruct[1].statusOpened = open
    devStruct[0].busy = false
    devStruct[1].busy = false
    updatePortStatus()
    if(open) {
        devStruct[0].settingsChanged = true
        devStruct[1].settingsChanged = true
        updateSerialPort(0)
        updateSerialPort(1)
    }
}