import './App.css';
import { Routes, Route, useNavigate } from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';
import Nav from '../Nav/Nav';
import { useEffect } from 'react';
import Login from '../Login/Login';
import Packets from '../Packets/Packets';
import { login, checkAuth } from '../../redux/actions/auth.action';
import Users from '../Users/Users';
import Device from '../Device/Device';
import InactiveDevices from '../Device/InactiveDevices';
import socket from '../../config/socket';
import { initInactiveDevices } from '../../redux/actions/devices.action'

function App() {
  const dispatch = useDispatch();
  const navigate = useNavigate();
  const { values } = useSelector((state) => state.user);

  useEffect(() => {
    const id = localStorage.getItem('id');
        if (id) {
      dispatch(login({ id }));
      dispatch(checkAuth());
      navigate('/packets');
    }
    else{
      dispatch(checkAuth());
      values.id? navigate('/packets'): navigate('/');
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [dispatch]);

  const handleInitInactiveDevices = (data) => {
    if (data) {
      dispatch(initInactiveDevices(data));
    }
  };

  useEffect(() => {
    socket.on('inactiveDevices', handleInitInactiveDevices);
  }, []);

  useEffect(() => {
    // Очищаем обработчики событий при размонтировании компонента
    return () => {
      socket.off('inactiveDevices');
    };
  }, []);

  return (
    <>
      <div className="wrapper">
        <header className="header">
          <Nav />
        </header>
        <main className="container">
          <Routes>
            <Route path="/" element={<Login />} />
            <Route path="/packets" element={<Packets />} />
            <Route path="/users" element={<Users />} />
            <Route path="/devices" element={<Device />} />
            <Route path="/inactive_devices" element={<InactiveDevices/>} />
          </Routes>
        </main>

        <footer></footer>
      </div>
    </>
  );
}

export default App;
