import React from 'react';
import Select from 'react-select';
import './CustomSelect.css'; 

const CustomSelect = ({ options, value, onChange, placeholder, width, height, disabled = false }) => {
  const customStyles = {
    control: (provided, state) => ({
      ...provided,
      ...(width && { width: width }),
      ...(height && { height: height }),
      marginRight: '10px',
      marginTop: '20px',
      boxShadow: state.isFocused ? 0 : 0,
      borderColor: state.isFocused
          ? 'rgba(0,0,0,0.1)' // Измените на нужный цвет или удалите для использования значения по умолчанию
          : provided.borderColor,
      "&:hover": {
        borderColor: state.isFocused
          ? 'rgba(0,0,0,0.1)' // Измените на нужный цвет или удалите для использования значения по умолчанию
          : provided.borderColor
      },
      ...(disabled && {
        backgroundColor: '#999', // Серый цвет фона для отключенного состояния
        color: '#ffffff', // Цвет текста для отключенного состояния
        cursor: 'not-allowed',
      })
    }),
    menu: (provided) => ({
      ...provided,
      zIndex: 1000 // Установите значение z-index выше, чем у таблицы
    }),
    option: (provided, state) => ({
      ...provided,
      backgroundColor: state.isSelected ? '#e4d3d2' : provided.backgroundColor,
      // Другие стили для выбранной опции
    }),
    placeholder: (provided, state) => ({
      ...provided,
      color: disabled ? '#ffffff' : '#7B817C', // Цвет текста плейсхолдера в зависимости от состояния
    }),
    singleValue: (provided, state) => ({
      ...provided,
      color: disabled ? '#ffffff' : '#7B817C', // Цвет текста выбранной опции в зависимости от состояния
    }),
  };

  return (
    <Select 
      options={options} 
      value={value} 
      onChange={onChange} 
      className="custom-select" 
      styles={customStyles} 
      placeholder={placeholder} 
      isDisabled={disabled} 
      isClearable={true}
    />
  );
};

export default CustomSelect;
