import React, { useState, forwardRef } from 'react';
import DatePicker from 'react-datepicker';
import 'react-datepicker/dist/react-datepicker.css';
import './DateRangePicker.css';
import moment from 'moment'


const DateRangePicker = ({ onChange, disabled }) => {
  const [dateRange, setDateRange] = useState([null, null]);
  const [startDate, endDate] = dateRange;

  const handleChange = (update) => {
    if (update) {
      const [start, end] = update;
      const startWithTime = start ? moment(start).set({ hour: 0, minute: 0, second: 0, millisecond: 0 }).toDate() : null;
      const endWithTime = end ? moment(end).set({ hour: 23, minute: 59, second: 59, millisecond: 999 }).toDate() : null;
      setDateRange([startWithTime, endWithTime]);
  
      if (onChange) {
        onChange(startWithTime, endWithTime);
      }
    }
  };


  const CustomInput = forwardRef(({ value, onClick }, ref) => {
    // Форматируем только те даты, которые были выбраны
    const formattedStartDate = startDate ? moment(startDate).format('DD.MM.YYYY') : '';
    const formattedEndDate = endDate ? moment(endDate).format('DD.MM.YYYY') : '';
    const formattedValue = startDate ? `${formattedStartDate} - ${formattedEndDate || 'Выберите дату'}` : '';
  
    return (
      <div className="custom-input-container-date" onClick={onClick} ref={ref}>
        <input 
          type="text" 
          className="custom-input-date" 
          value={formattedValue} 
          readOnly 
          placeholder="Выберите диапазон дат" 
          disabled={disabled} 
        />
      </div>
    );
  });

  return (
    <DatePicker
      selectsRange={true}
      startDate={startDate}
      endDate={endDate}
      onChange={handleChange}
      isClearable={true}
      customInput={<CustomInput />}
      disabled={disabled}
    />
  );
};

export default DateRangePicker;
