import React, { useState, useRef, useEffect } from 'react';

const DateTimeRangePicker = ({ onChange }) => {
  const [startDate, setStartDate] = useState('');
  const [endDate, setEndDate] = useState('');
  const startInputRef = useRef(null);
  const endInputRef = useRef(null);

  const handleChangeStart = (event) => {
    setStartDate(event.target.value);
    onChange({ start: event.target.value, end: endDate });
  };

  const handleChangeEnd = (event) => {
    setEndDate(event.target.value);
    onChange({ start: startDate, end: event.target.value });
  };

  const handleFocus = (ref) => {
    ref.current.type = 'datetime-local';
  };

  const handleBlur = (ref, dateSetter) => {
    ref.current.type = 'text';
    if (!ref.current.value) {
      dateSetter('');
    }
  };

  return (
    <div>
      <input
        ref={startInputRef}
        type="text"
        className="custom-input-date"
        value={startDate}
        placeholder="Начало диапазона"
        onFocus={() => handleFocus(startInputRef)}
        onBlur={() => handleBlur(startInputRef, setStartDate)}
        onChange={handleChangeStart}
      />
        <input
          ref={endInputRef}
          type="text"
          className="custom-input-date"
          value={endDate}
          placeholder="Конец диапазона"
          onFocus={() => handleFocus(endInputRef)}
          onBlur={() => handleBlur(endInputRef, setEndDate)}
          onChange={handleChangeEnd}
        />
    </div>
  );
};

export default DateTimeRangePicker;
