import React from 'react';
import './Device.css';
import CustomSelect from '../CustomElements/CustomSelect';
import CustomInput from '../CustomElements/CustomInput';
import CustomModal from '../Modal/CustomModal';
import { useState, useEffect } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { initDevicesFromServer, editDeviceFromServer } from '../../redux/actions/devices.action';
import Table from '../Table/Table';
import EditDevice from './EditDevice';

function Device(props) {
  const dispatch = useDispatch();
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [isModalOpenEditDevice, setIsModalOpenEditDevice ] = useState(false)
  const devices = useSelector((state) => state.devices.forEditDevices);
  const allDevices = useSelector((state) => state.devices.allDevices);
  const [spsSeriesID, setSpsSeriesID] = useState('');
  const [spsNumber, setSpsNumber] = useState('');
  const [spsSection, setSpsSection] = useState('');
  const [litera, setLitera] = useState('');
  const [selectedOption, setSelectedOption] = useState(null);
  const [options, setOptions] = useState([]);
  const [currentDevice, setCurrentDevice] = useState()
  //состояние для проверки валидности формы
  const [isFormValid, setIsFormValid] = useState(false);

  const resetForm = () => {
    setSpsSeriesID('');
    setSpsNumber('');
    setSpsSection('');
    setLitera('');
    setSelectedOption(null);
  };

  useEffect(() => {
    function createOptions(devices) {
      if (!devices) {
        return [];
      }
      return devices.map((device) => ({
        value: device.uuid,
        label: device.uuid,
      }));
    }

    setOptions(createOptions(devices));
  }, [devices]);

  const handleSubmit = (e) => {
    e.preventDefault();
    const payload = {
      spsSeriesID,
      spsNumber,
      spsSection,
      litera,
      isAuthorized: true,
      uuid: selectedOption ? selectedOption.value : null,
    };
    dispatch(editDeviceFromServer(payload));
    resetForm();
    setIsModalOpen(true)
  };

  useEffect(() => {
    dispatch(initDevicesFromServer());
  }, []);

  useEffect(() => {
    setIsFormValid(spsSeriesID && spsNumber && spsSection && litera && selectedOption);
  }, [spsSeriesID, spsNumber, spsSection, litera, selectedOption]);

  const handleClose = () => {
    setIsModalOpen(false);
    setIsModalOpenEditDevice(false)
  };

  const handleRowButtonClick = (rowData) => {
    setIsModalOpenEditDevice(true)
    setCurrentDevice(rowData)
  }

  return (
    <>
      <form className="new-entity-form" onSubmit={handleSubmit}>
        <div className="device-custom-select_container">
          <CustomSelect
            options={options}
            value={selectedOption}
            onChange={setSelectedOption}
            placeholder={'Выберите ID устройства'}
            width={'420px'}
            height={'50px'}
          />
        </div>
        <div className="device-input-container">
          <div className="device-input-wrap">
            <CustomInput label="Серия СПС:" value={spsSeriesID} onChange={(e) => setSpsSeriesID(e.target.value)} required />
            <CustomInput label="Заводской номер СПС:" value={spsNumber} onChange={(e) => setSpsNumber(e.target.value)} required />
          </div>
          <div className="device-input-wrap">
            <CustomInput label="Номер секции СПС:" value={spsSection} onChange={(e) => setSpsSection(e.target.value)} required />
            <CustomInput label="СПС Литера:" value={litera} onChange={(e) => setLitera(e.target.value)} required />
          </div>
        </div>

        <div className="device-btn-container">
          <button
            className={`device-btn uk-button ${isFormValid ? 'uk-button-secondary' : 'uk-button-disabled'}`}
            type="submit"
            disabled={!isFormValid}
          >
            Связать
          </button>
        </div>
      </form>
      <CustomModal
        isOpen={isModalOpen}
        content={<div className="device-message-success">Данные успешно добавлены</div>}
        onClose={handleClose}
        showCloseButton={true}
        width={400} // Заданная ширина
        height={150} // Заданная высота
      />
      <div className="styled-table-wrapper">
        <h5 className="device-title">Данные по авторизованым устройствам, для редактирования данных кликните по ID устройства</h5>
        <Table data={allDevices} onRowButtonClick={(rowData) => handleRowButtonClick(rowData)}/>
      </div>
      <CustomModal
        isOpen={isModalOpenEditDevice}
        content={<EditDevice currentDevice={currentDevice} clouseModal={handleClose} />}
        onClose={handleClose}
        showCloseButton={true}
        width={700} // Заданная ширина
        height={600} // Заданная высота
      />
    </>
  );
}

export default Device;
