import { useEffect, useState} from 'react';
import './Device.css';
import CustomInput from '../CustomElements/CustomInput';
import Modal from '../Modal/Modal';
import { useDispatch } from 'react-redux';
import { editDeviceFromServer } from '../../redux/actions/devices.action';

function EditDevice({currentDevice, clouseModal}) {
  const dispatch = useDispatch();
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [spsSeriesID, setSpsSeriesID] = useState(currentDevice.spsSeriesID);
  const [spsNumber, setSpsNumber] = useState(currentDevice.spsNumber);
  const [spsSection, setSpsSection] = useState(currentDevice.spsSection);
  const [litera, setLitera] = useState(currentDevice.litera);
  //состояние для проверки валидности формы
  const [isFormValid, setIsFormValid] = useState(false);

  useEffect(() => {
    setIsFormValid(spsSeriesID && spsNumber && spsSection && litera);
  }, [spsSeriesID, spsNumber, spsSection, litera, ]);

  const handleSubmit = (e) => {
    e.preventDefault();
    const payload = {
      spsSeriesID,
      spsNumber,
      spsSection,
      litera,
      isAuthorized: true,
      uuid: currentDevice.uuid,
    };
    dispatch(editDeviceFromServer(payload));
    setIsModalOpen(true)
  };

  const handleClick = (e) => {
    e.preventDefault();
    const payload = {
      spsSeriesID: '',
      spsNumber: '',
      spsSection: '',
      litera: '',
      isAuthorized: false,
      uuid: currentDevice.uuid,
    }
    dispatch(editDeviceFromServer(payload));
    
    setIsModalOpen(true)
  }

  const onClouseModal = () => {
    setIsModalOpen(false);
    clouseModal()
  };

  return (
    <>
      <form className="new-entity-form">
        <div className="device-custom-select_container">
          <CustomInput label="ID устройства" value={currentDevice.uuid} isDisabled={true}/>
        </div>
        <div className="device-input-container">
          <CustomInput label="Серия СПС" value={spsSeriesID} onChange={(e) => setSpsSeriesID(e.target.value)} required />
          <CustomInput label="Заводской номер СПС" value={spsNumber} onChange={(e) => setSpsNumber(e.target.value)} required />
          <CustomInput label="Номер секции СПС" value={spsSection} onChange={(e) => setSpsSection(e.target.value)} required />
          <CustomInput label="СПС Литера" value={litera} onChange={(e) => setLitera(e.target.value)} required />
        </div>
        <div className="device-btn-container">
          <button
            className={`device-btn uk-button ${isFormValid ? 'uk-button-secondary' : 'uk-button-disabled'}`}
            onClick={handleSubmit}
            disabled={!isFormValid}
          >
            Изменить
          </button>
          <button
            className={`device-btn uk-button ${isFormValid ? 'uk-button-secondary' : 'uk-button-disabled'}`}
            onClick={handleClick}
            disabled={!isFormValid}
          >
            Сбросить данные
          </button>
        </div>
      </form>
      {isModalOpen && <Modal onClouseModal={onClouseModal} text="Данные успешно изменены" />}
    </>
  );
}

export default EditDevice;
