/* eslint-disable react-hooks/exhaustive-deps */
import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { useSelector, useDispatch } from 'react-redux';
import { initPacketsFromServer } from '../../redux/actions/packets.action';
import Table from '../Table/Table';
import generateOptions from '../../utils/generateOptions';
import CustomSelect from '../CustomElements/CustomSelect';
import './Packets.css';
import CustomButton from '../CustomElements/CustomButton';
import TabSwitch from '../CustomElements/TabSwitch';
import DateRangePicker from '../CustomElements/DateRangePicker.jsx';
import { initDevicesFromServer } from '../../redux/actions/devices.action';
import moment from 'moment'

function Packets() {
  // Инициализация Redux диспетчера и навигации
  const dispatch = useDispatch();
  const navigate = useNavigate();

  // Локальное состояние компонента
  const [currentPage, setCurrentPage] = useState(0);
  const [sortDt, setSortDt] = useState(true);
  const [trigger, setTrigger] = useState(true);
  const [optionsDevices, setOptionsDevices] = useState([]);

  // Создаем два состояния для хранения начальной и конечной дат
  const [selectedStartDate, setSelectedStartDate] = useState(null);
  const [selectedEndDate, setSelectedEndDate] = useState(null);

  useEffect(() => {
    console.log(selectedEndDate, selectedStartDate);
  }, [selectedEndDate, selectedStartDate]);

  // Получение данных из Redux store
  const { values } = useSelector((state) => state.user);
  const { packets, totalPages } = useSelector((state) => state.packets);
  const devices = useSelector((state) => state.devices.forEditDevices);
  const allDevices = useSelector((state) => state.devices.allDevices);

  // Локальные состояния для хранения выбранных опций в селектах
  const [selectedOption, setSelectedOption] = useState(null);
  const [selectedOptionDevices, setSelectedOptionDevices] = useState(null);
  const options = generateOptions();

  // useEffect для создания опций устройств
  useEffect(() => {
    function createOptions(devices) {
      if (!devices) {
        return [];
      }
      return devices.map((device) => ({
        value: device.uuid,
        label: device.uuid,
      }));
    }

    setOptionsDevices(createOptions([...devices, ...allDevices]));
  }, [devices]);

  // Обработчики изменений в различных элементах управления интерфейса
  const handleChange = (option) => {
    setSelectedOption(option);
  };

  const handleChangeDevices = (option) => {
    setSelectedOptionDevices(option);
  };

  const handleChangeSortDt = (value) => {
    setSortDt(value);
    setCurrentPage(1);
    setTrigger((prevBoolean) => !prevBoolean);
  };

  const handleChangeRefreshPage = () => {
    setCurrentPage(1);
    setTrigger((prevBoolean) => !prevBoolean);
  };

  // Навигация по страницам
  const handleChangeNextPage = () => {
    if (currentPage < totalPages) {
      setCurrentPage((prev) => prev + 1);
    }
  };

  const handleChangePreviousPage = () => {
    if (currentPage >= 2) {
      setCurrentPage((prev) => prev - 1);
    }
  };

  // Обновление пакетов при изменении текущей страницы или триггера
  useEffect(() => {
    if (selectedOption) {
      let payload = {
        currentPage,
        sortDt,
        packetNum: selectedOption?.value,
        deviceId: selectedOptionDevices?.value || false,
        startDate: selectedStartDate,
        endDate: selectedEndDate,
      };
      dispatch(initPacketsFromServer(payload));
    }
  }, [currentPage, trigger]);

  // Обновление пакетов при изменении выбранной опции пакетов
  useEffect(() => {
    if (selectedOption) {
      let payload = {
        currentPage: 1,
        sortDt,
        packetNum: selectedOption?.value,
        deviceId: selectedOptionDevices?.value || false,
        startDate: selectedStartDate,
        endDate: selectedEndDate,
      };
      dispatch(initPacketsFromServer(payload));
      setCurrentPage(1);
    }
  }, [selectedOption]);

  // Обновление пакетов при изменении выбранной опции ID устройств
  useEffect(() => {
    if (selectedOption) {
      let payload = {
        currentPage: 1,
        sortDt,
        packetNum: selectedOption?.value,
        deviceId: selectedOptionDevices?.value || false,
        startDate: selectedStartDate,
        endDate: selectedEndDate,
      };
      dispatch(initPacketsFromServer(payload));
      setCurrentPage(1);
    }
  }, [selectedOptionDevices]);

    // Обновление пакетов при изменении диапазона дат
    useEffect(() => {
      if (selectedOption && selectedStartDate && selectedEndDate) {
        let payload = {
          currentPage: 1,
          sortDt,
          packetNum: selectedOption?.value,
          deviceId: selectedOptionDevices?.value || false,
          startDate: selectedStartDate,
          endDate: selectedEndDate,
        };
        dispatch(initPacketsFromServer(payload));
        setCurrentPage(1);
      }
    }, [selectedStartDate, selectedEndDate]);

  // Перенаправление на главную страницу, если пользователь не авторизован
  useEffect(() => {
    if (!values.id) {
      navigate('/');
    }
  }, [values]);

  // Инициализация устройств при монтировании компонента
  useEffect(() => {
    dispatch(initDevicesFromServer());
  }, []);

  // Обработка изменения диапазона дат
  const handleDateRangeChange = (startDate, endDate) => {
    const startInUtc = startDate ? moment.utc(startDate).format() : null;
    const endInUtc = endDate ? moment(endDate).utc().format() : null;
  
    setSelectedStartDate(startInUtc);
    setSelectedEndDate(endInUtc);
  };

  return (
    <>
      <div className="select-container">
        <CustomSelect
          options={options}
          value={selectedOption}
          onChange={handleChange}
          placeholder={'Выберите номер пакета'}
          width="250px"
          height={'50px'}
        />
        <DateRangePicker onChange={handleDateRangeChange} disabled={!selectedOption}/>
        <CustomSelect
          options={[{ value: false, label: 'Все устройства' }, ...optionsDevices]}
          value={selectedOptionDevices}
          onChange={handleChangeDevices}
          placeholder={'Выберите ID устройства'}
          width="250px"
          height={'50px'}
          disabled={!selectedOption}
        />
        <TabSwitch disabled={!selectedOption} onClick={handleChangeSortDt} />
        <CustomButton text={'Обновить'} onClick={handleChangeRefreshPage} disabled={!selectedOption} width="110px" />
        <CustomButton text={'Предыдущая'} onClick={handleChangePreviousPage} disabled={!selectedOption} width="110px" />
        <div className="packets-text">
          {currentPage}/{totalPages ? totalPages : 0}
        </div>
        <CustomButton text={'Следующая'} onClick={handleChangeNextPage} disabled={!selectedOption} width="110px" />
      </div>
      <div className="styled-table-wrapper">
        <Table data={packets} />
      </div>
    </>
  );
}

export default Packets;
