  /**
 * Перемещает устройство из allDevices в forEditDevices.
 *
 * @param {Object} state - Текущее состояние, содержащее объекты allDevices и forEditDevices.
 * @param {Object} payload - Объект, содержащий uuid устройства для перемещения.
 * @returns {Object} Обновленное состояние с модифицированными массивами allDevices и forEditDevices.
 */
  export function moveDeviceToEdit(state, payload) {
    const { uuid } = payload;
  
    // Находим и удаляем устройство из allDevices
    const updatedAllDevices = state.allDevices.filter(device => device.uuid !== uuid);
  
    // Проверяем, существует ли уже устройство в forEditDevices
    const deviceExistsInForEdit = state.forEditDevices.some(device => device.uuid === uuid);
  
    // Если устройства нет в forEditDevices, добавляем его туда
    let updatedForEditDevices = state.forEditDevices;
    if (!deviceExistsInForEdit) {
      const deviceToAdd = state.allDevices.find(device => device.uuid === uuid);
      if (deviceToAdd) {
        updatedForEditDevices = [deviceToAdd, ...state.forEditDevices];
      }
    }
  
    return {
      ...state,
      allDevices: updatedAllDevices,
      forEditDevices: updatedForEditDevices
    };
  }
