/**
 * Обновляет массивы устройств в состоянии приложения.
 * 
 * @param {Object} state - Текущее состояние, содержащее объект devices.
 * @param {Object} updatedDevice - Устройство, которое нужно обновить или добавить.
 * @returns {Object} Обновленное состояние с модифицированными массивами allDevices и forEditDevices.
 */
export function updateDevices(state, updatedDevice) {
  // Поиск устройства в allDevices и его обновление или добавление
  const deviceExistsInAllDevices = state.allDevices.find(device => device.uuid === updatedDevice.uuid);
  let updatedAllDevices;
  if (deviceExistsInAllDevices) {
    updatedAllDevices = state.allDevices.map(device => device.uuid === updatedDevice.uuid ? updatedDevice : device);
  } else {
    // Добавляем в начало массива
    updatedAllDevices = [updatedDevice, ...state.allDevices];
  }

  // Удаление устройства из forEditDevices, если оно там есть
  const updatedForEditDevices = state.forEditDevices.filter(device => device.uuid !== updatedDevice.uuid);

  return {
    ...state, // Копируем остальные части состояния
    allDevices: updatedAllDevices,
    forEditDevices: updatedForEditDevices
  };
}
