/* eslint-disable consistent-return */
/* eslint-disable no-param-reassign */
const { authorization } = require('../services/authorization.service');
const { deleteRefreshToken } = require('../services/token.service');

const logger = require('../utils/logger');

const loginWs = (io, socket) => {
  socket.on('authenticate', async (credentials) => {
    try {
      const user = await authorization(credentials);
      socket.emit('authReply', user);
    } catch (error) {
      socket.emit('authReply', { status: 'error' });
      logger.error('Error during authentication:', error);
    }
  });
};

const logoutWs = (io, socket) => {
  socket.on('logout', async () => {
    try {
      // socket.disconnect();
    } catch (error) {
      logger.error('Error during logout:', error);
    }
  });
};

const loginHttp = async (req, res) => {
  try {
    const credentials = req.body;
    const userData = await authorization(credentials);
    // Сохраняем refreshToken в куках
    res.cookie('refreshToken', userData.refreshToken, {
      maxAge: 1000 * 60 * 60 * 24 * 30, // Время жизни куки
      httpOnly: true, // Флаг, указывающий, что куки недоступны для скриптов на стороне клиента
    });
    res.cookie('accessToken', userData.accessToken, {
      maxAge: 1000 * 60 * 60,
      httpOnly: true,
    });
    delete userData.refreshToken;
    return res.json(
      userData,
    );
  } catch (error) {
    logger.error('Error during login from HTTP:', error);
    res.status(401)
      .json({
        message: error.message,
      }).end();
  }
};

const logoutHttp = async (req, res) => {
  try {
    const token = req.cookies.refreshToken;
    deleteRefreshToken(token);
    res.clearCookie('refreshToken');
    res.clearCookie('accessToken');
    logger.info('User logged out successfully, cookies cleared');
    res.status(200).end();
  } catch (error) {
    logger.error('Error during logout: ', error);
    res.status(500).send('Internal Server Error');
  }
};

const checkAuth = async (req, res) => {
  try {
    const { id } = req.user;
    const { accessToken } = req.cookies;

    if (id) {
      logger.info(`User ID ${id} is authenticated`);
      res.status(200).json({ id, accessToken, status: 'ok' });
    } else {
      logger.warn('User ID is not available');
      res.status(401).json({ error: 'Unauthorized' });
    }
  } catch (error) {
    logger.error('Error during auth check: ', error);
    res.status(500).json({ status: 'Internal Server Error' });
  }
};

module.exports = {
  loginWs, logoutWs, loginHttp, logoutHttp, checkAuth,
};
