/* eslint-disable consistent-return */
const {
  initDevicesFromServer,
  editDevicesFromServer,
  checkLastPacketReceivedFromDb,
} = require('../services/devices.services');

const logger = require('../utils/logger');

const initDevices = async (req, res) => {
  try {
    const devicesData = await initDevicesFromServer();
    return res.json(devicesData);
  } catch (error) {
    logger.error('Error init packets', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const editDevices = async (req, res) => {
  const payload = req.body;
  try {
    const devicesData = await editDevicesFromServer(payload);
    return res.json(devicesData);
  } catch (error) {
    logger.error('Error edit packets', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const checkLastPacketReceived = async (io) => {
  try {
    const devices = await checkLastPacketReceivedFromDb();
    if (devices.length >= 0) {
      io.emit('inactiveDevices', devices);
      logger.info('Inactive devices reported to the frontend.');
    }
  } catch (error) {
    logger.error(`Error during the check of last packets received: ${error}`);
  }
};

module.exports = { initDevices, editDevices, checkLastPacketReceived };
