/* eslint-disable consistent-return */
const {
  initPacketsFromServer,
  createPacketsFromServer,
  updatePacketsFromServer,
  removePacketsFromServer,
  postInitPacketsFromServer,
} = require('../services/packets.service');

const logger = require('../utils/logger');

const initPackets = async (req, res) => {
  try {
    const payload = req.body;
    const packetsData = await initPacketsFromServer(payload);
    return res.json(packetsData);
  } catch (error) {
    logger.error('Error init packets', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const createPackets = (io, socket) => {
  socket.on('createPackets', async (payload) => {
    try {
      const packets = await createPacketsFromServer(payload);
      socket.emit('createPacketsSuccsses', packets);
    } catch (error) {
      logger.error('Error create packets', error);
    }
  });
};

const editPackets = async (req, res) => {
  const payload = req.body;
  try {
    const packetsData = await updatePacketsFromServer(payload);
    return res.json(packetsData);
  } catch (error) {
    logger.error('Error edit packets', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const removePackets = async (req, res) => {
  const { id } = req.body;
  try {
    const packetsData = await removePacketsFromServer(id);
    return res.json(packetsData);
  } catch (error) {
    logger.error('Error remove packets', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const postInitPackets = async (req, res) => {
  const payload = req.body;
  try {
    const packetsData = await postInitPacketsFromServer(payload);
    return res.json(packetsData);
  } catch (error) {
    logger.error('Error during packet initialization', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

module.exports = {
  initPackets,
  createPackets,
  editPackets,
  removePackets,
  postInitPackets,
};
