/* eslint-disable consistent-return */
const {
  initUserFromServer,
  createUserFromServer,
  updateUserFromServer,
  removeUserFromServer,
} = require('../services/user.service');

const logger = require('../utils/logger');

const initUser = async (req, res) => {
  try {
    const userData = await initUserFromServer();
    return res.json(userData);
  } catch (error) {
    logger.error('Error init user', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const createUser = async (req, res) => {
  const payload = req.body;
  try {
    const userData = await createUserFromServer(payload);
    return res.json(userData);
  } catch (error) {
    logger.error('Error create user', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const editUser = async (req, res) => {
  const payload = req.body;
  try {
    const userData = await updateUserFromServer(payload);
    return res.json(userData);
  } catch (error) {
    logger.error('Error edit user', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

const removeUser = async (req, res) => {
  const { id } = req.body;
  try {
    const userData = await removeUserFromServer(id);
    return res.json(userData);
  } catch (error) {
    logger.error('Error remove user', error);
    res.status(500)
      .json({
        message: error.message,
      }).end();
  }
};

module.exports = {
  initUser,
  createUser,
  editUser,
  removeUser,

};
