require('dotenv').config();
const tunnel = require('tunnel-ssh');

// initialize database connection
const mongoose = require('mongoose');
const logger = require('../utils/logger');

if (process.env.LOCAL) {
  const config = {
    username: process.env.USER_DB,
    host: process.env.HOST_DB,
    agent: process.env.SSH_AUTH_SOCK,
    port: 2022,
    dstPort: 27017,
    password: process.env.PASSWORD_DB,
  };

  // eslint-disable-next-line no-unused-vars, no-shadow
  const server = tunnel(config, (error, server) => {
    if (error) {
      logger.error(`SSH connection error: ${error}`);
    }
    mongoose.connect(process.env.CONNECTION_STRING);

    const db = mongoose.connection;
    db.on('error', console.error.bind(console, 'DB connection error:'));
    db.once('open', () => {
      logger.info('DB connection successful');
    });
  });
} else {
// подключение к бд через CONNECTION_STRING
  const connectionString = process.env.CONNECTION_STRING;
  if (!connectionString) {
    logger.error('MongoDB connection string missing!');
    process.exit(1);
  }
  mongoose.connect(connectionString, {
    useNewUrlParser: true,
  });
  const db = mongoose.connection;
  db.on('error', (err) => {
    logger.error(`MongoDB error: ${err.message}`);
    process.exit(1);
  });
  db.once('open', () => logger.info('MongoDB connection established'));
}
