const logger = require('../utils/logger');
const { userObj } = require('../utils/user.dto');
const {
  generateTokens,
  isAccessTokenValid,
  isRefreshTokenValid,
  storeRefreshToken,
  findStoredRefreshToken,
} = require('../services/token.service');

// доделать достать токен при евентах и верефицировать

function verifyTokenWs(socket, next) {
  socket.use((packet, next) =>
    // if (packet[0] !== 'authenticate' && packet[0] !== 'logout') {
    //   const token = packet[1]?.token;
    //   if (!token || !isAccessTokenValid(token)) {
    //     return next(new Error('Authentication error'));
    //   }
    // }
    next());
}

const verifyTokenHttp = async (req, res, next) => {
  const { accessToken, refreshToken } = req.cookies;

  // Если токенов нет, возвращаем ошибку
  if (!accessToken && !refreshToken) {
    logger.error('Tokens are missing');
    return res.status(403).send({ message: 'Tokens are required for authentication' });
  }
  try {
    // Проверяем, валиден ли токен доступа
    if (accessToken && isAccessTokenValid(accessToken)) {
      req.user = isAccessTokenValid(accessToken);
      return next();
    }

    logger.warn('Access token is invalid or expired');

    // Если токен доступа невалиден или отсутствует, проверяем рефреш токен
    if (refreshToken) {
      const refreshTokenPayload = isRefreshTokenValid(refreshToken);
      const refreshTokenFromDb = await findStoredRefreshToken(refreshToken);

      // Если рефреш токен валиден, генерируем новую пару токенов
      if (refreshTokenPayload && refreshTokenFromDb) {
        const payload = userObj(refreshTokenPayload);
        const newTokens = generateTokens(payload);

        // Сохраняем новый рефреш токен в базе данных
        await storeRefreshToken(refreshTokenPayload.id, newTokens.refreshToken);
        // Устанавливаем новые токены в куки
        res.cookie('accessToken', newTokens.accessToken, {
          maxAge: 1000 * 60 * 60,
          httpOnly: true,
        });
        res.cookie('refreshToken', newTokens.refreshToken, {
          maxAge: 1000 * 60 * 60 * 24 * 30,
          httpOnly: true,
        });
        req.user = refreshTokenPayload;
        logger.info('new tokens generated successfully');
        return next();
      }
    }
    logger.error('Refresh token is invalid');
    return res.status(401).send({ message: 'Invalid tokens' });
  } catch (error) {
    // Логгируем и возвращаем ошибку, если что-то пошло не так
    logger.error('Error during token verification:', error);
    return res.status(500).send({ message: 'Internal server error' });
  }
};

// const verifyTokenHttp = (req, res, next) => {
//   const token = req.cookies.accessToken;

//   if (!token) {
//     return res.status(403).send({ message: 'A token is required for authentication' });
//   }
//   try {
//     // Проверяем валидность токена
//     const decoded = isAccessTokenValid(token);
//     const { id, login } = decoded;
//     // Сохраняем декодированные данные токена в объекте запроса,
//     // чтобы они были доступны в последующих обработчиках
//     req.user = decoded;
//   } catch (err) {
//     return res.status(401).send({ message: 'Invalid token' });
//   }

//   // Если токен валидный, передаем управление следующему middleware или обработчику маршрута
//   return next();
// };

module.exports = { verifyTokenWs, verifyTokenHttp };
