const bcrypt = require('bcrypt');
const User = require('../db/models/user');
const logger = require('../utils/logger');
const { userObj } = require('../utils/user.dto');
const { generateTokens, storeRefreshToken } = require('./token.service');

const authorization = async (credentials) => {
  try {
    const { login, password } = credentials;
    // Находим пользователя по username
    const currentUser = await User.findOne({ login });
    if (!currentUser?.login) {
      return { success: false, error: `Пользователь с ${login} не зарегистрирован!`, status: 'error' };
    }
    const validPassword = await bcrypt.compare(password, currentUser.password);
    if (!validPassword) {
      return { success: false, error: 'Пароль не верный', status: 'error' };
    }
    // убираем из объекта пароль
    const userToken = userObj(currentUser);
    // генерируем пару токенов
    const tokens = generateTokens({ ...userToken });
    // сохраняем рефреш токены в бд
    await storeRefreshToken(userToken.id, tokens.refreshToken);
    // Если все хорошо, возвращаем объект пользователя
    return { id: currentUser.id, ...tokens, status: 'ok' };
  } catch (error) {
    // Ловим ошибку и прокидываем ее дальше, чтобы обработать на уровне контроллера
    logger.error('Error during authentication:', error); // Логгирование ошибки
    throw error;
  }
};

module.exports = {
  authorization,

};
