// Импорт моделей и утилит
// eslint-disable-next-line import/no-extraneous-dependencies
const moment = require('moment');
const Packets = require('../db/models/packets');
const Devices = require('../db/models/devices');
const logger = require('../utils/logger');

// Функция для вычисления CRC16
function crc16(buffer) {
  var crc = 0xFFFF;
  var odd;

  for (var i = 0; i < buffer.length; i++) {
    crc = crc ^ buffer[i];

    for (var j = 0; j < 8; j++) {
      odd = crc & 0x0001;
      crc = crc >> 1;
      if (odd) {
        crc = crc ^ 0xA001;
      }
    }
  }

  crc = ((crc << 8) & 0xFF00) | ((crc >> 8) & 0x00FF);

  return crc;
};

// Функция для преобразования hex-строки в Uint8Array
function hexToUint8Array(hexString) {
  if (hexString.length % 2 !== 0) {
    throw new Error('Invalid hex string length');
  }

  const arrayBuffer = new Uint8Array(hexString.length / 2);

  for (let i = 0; i < hexString.length; i += 2) {
    const byteValue = parseInt(hexString.substr(i, 2), 16);
    if (isNaN(byteValue)) {
      throw new Error('Invalid hex string');
    }
    arrayBuffer[i / 2] = byteValue;
  }

  return arrayBuffer;
}

function convertModbusHexToFloat(hexString) {
  console.log("Original Hex String:", hexString);

  if (hexString.length !== 8) {
    throw new Error('Hex string must be 8 characters long');
  }

  // Rearrange the hex string from '999a428f' to '428f999a' (big-endian)
  const reorderedHexString = hexString.substring(4, 8) + hexString.substring(0, 4);

  const int = parseInt(reorderedHexString, 16);

  // Calculate the sign, exponent, and fraction from the integer
  const sign = (int >>> 31) ? -1 : 1;
  const exponent = ((int >>> 23) & 0xFF) - 127;
  const fraction = ((int & 0x7FFFFF) + 0x800000) / 0x800000;

  // Calculate the float from the sign, exponent, and fraction
  const float = sign * fraction * Math.pow(2, exponent);

  return float.toFixed(2);
}
function convertModbusHexToUint32(hexString) {


  // if (hexString.length !== 4 && hexString.length !== 8) {
  //   throw new Error('Hex string must be 4 or 8 characters long');
  // }

  // Rearrange the hex string from little-endian to big-endian
  const reorderedHexString = hexString;

  // Convert to uint32
  const uint32 = parseInt(reorderedHexString, 16);
  console.log("Original Hex String:", hexString, " int ", uint32);

  return uint32;
}
function convertInvModbusHexToUint32(hexString) {


  // if (hexString.length !== 4 && hexString.length !== 8) {
  //   throw new Error('Hex string must be 4 or 8 characters long');
  // }

  // Rearrange the hex string from little-endian to big-endian
  const reorderedHexString = hexString.substring(4, 8) + hexString.substring(0, 4);

  // Convert to uint32
  const uint32 = parseInt(reorderedHexString, 16);
  console.log("Original Hex String:", hexString, " int ", uint32);

  return uint32;
}

// Функция для вычисления CRC16
function isValidCRC16(hexString) {
  // Проверка на наличие hexString
  if (!hexString) {
    return false;
  }
  const arrayBuffer = hexToUint8Array(hexString);
  const trimmedArrayBuffer = arrayBuffer.slice(0, -2);
  const crc = crc16(trimmedArrayBuffer);
  const crcHexString = crc.toString(16).toUpperCase();
  const crcValue = hexString.slice(-4).toUpperCase();

  return crcValue === crcHexString;
}

function parsePackets(packets) {
  const modifiedPackets = packets.map(item => {
    let plainObject = item;
    delete plainObject.pack_lon;
    delete plainObject.pack_lat;
    delete plainObject.speed;
    delete plainObject.voltage;
    delete plainObject.geo;
    delete plainObject.timestamp;
    delete plainObject.unix_time;
    delete plainObject.data_type;

    if (plainObject.packetNum == 56) {
      delete plainObject["level_h2o"];
      delete plainObject["volume"];
      delete plainObject["volume_oil"];
      delete plainObject["mass"];
      delete plainObject["mass_oil"];
    }
    if (plainObject.packetNum == 50) {
      delete plainObject["fuel_used_hour"];
      delete plainObject["fuel_used_total"];
      delete plainObject["fuel_used_hour_average"];
      delete plainObject["fuel_used_hour_feed"];
      delete plainObject["fuel_used_hour_return"];``
      delete plainObject["fuel_used_total_hr"];
      delete plainObject["chamber_runtime_idle_feed"];
      delete plainObject["chamber_runtime_optimal_feed"];
      delete plainObject["chamber_runtime_overload_feed"];
      delete plainObject["chamber_runtime_cheat_feed"];
      delete plainObject["chamber_runtime_return"];
    }

    Object.keys(plainObject).forEach(key => {
      if (key.endsWith('_hex')) {
        // Extract the base field name
        const baseField = key.slice(0, -4); // Removes the '_hex' part

        // Check if the corresponding base field exists
        if (plainObject.hasOwnProperty(baseField)) {
          // Validate the CRC16 of the hex field
          if (!isValidCRC16(plainObject[key])) {
            // If CRC is invalid, set the base field to 'n/a'
            //plainObject[baseField] = (plainObject.packetNum == 50) ? 'n/a' : '';
            delete plainObject[key];
            delete plainObject[baseField];
          }
          else {
            if (plainObject.packetNum == 56) {
              switch (baseField) {
                case "level_oil":
                case "temp_oil":
                case "density_oil":
                case "status":
                case "error_status":
                  if (plainObject[key].length >= 12) {
                    const buffer = plainObject[key].slice(-12, -4)
                    const value = convertModbusHexToFloat(buffer);
                    console.log(buffer, " - ", value);
                    plainObject[baseField] = value;
                  }
                  else {
                    plainObject[baseField] = '';
                  }
                  break;
                case "level_h2o":
                case "volume":
                case "volume_oil":
                case "mass":
                case "mass_oil":
                  delete plainObject[baseField];
                  break;
              }
            }
            if (plainObject.packetNum == 50) {
              switch (baseField) {
                case "fuel_used_hour":
                case "fuel_used_total":
                // {
                // if (plainObject[baseField] > 0) {
                //   const buffer = plainObject[key].slice(-12, -4)
                //   const value = convertModbusHexToUint32(buffer);
                //   plainObject[baseField] = (value * 0.001).toFixed(1);  
                // }
                // }
                case "fuel_used_hour_average":
                case "fuel_used_hour_feed":
                case "fuel_used_hour_return": //{
                // if (plainObject[baseField] > 0) {
                //   const buffer = plainObject[key].slice(-12, -4)
                //   const value = convertModbusHexToUint32(buffer);
                //   plainObject[baseField] = (value * 0.05).toFixed(2);

                // }
                //}
                case "fuel_used_total_hr":
                  // const buffer = plainObject[key].slice(-12, -4)
                  // const value = convertInvModbusHexToUint32(buffer) * 0.001
                  // plainObject[baseField] = value.toFixed(1);
                  delete plainObject[baseField];
                  break;

                case "fuel_temperature": {
                  const buffer = plainObject[key].slice(-12, -4)
                  const value = convertInvModbusHexToUint32(buffer) - 40;
                  console.log("TEMP: ", buffer, "VALUE: ", value)
                  plainObject[baseField] = value.toFixed(1);
                  break;
                }
                // из 52 пакета
                case "chamber_runtime_idle_feed":
                case "chamber_runtime_optimal_feed":
                case "chamber_runtime_overload_feed":
                case "chamber_runtime_cheat_feed":
                case "chamber_runtime_return":
                  delete plainObject[baseField];
                  break;
                case "fuel_used_total_feed": {
                  const buffer = plainObject[key].slice(-12, -4)
                  const value = convertInvModbusHexToUint32(buffer) * 0.001
                  plainObject[baseField] = value.toFixed(1);
                  break;
                }
                case "chamber_runtime_feed": {
                  const buffer = plainObject[key].slice(-12, -4)
                  const value = convertInvModbusHexToUint32(buffer) * 0.1
                  plainObject[baseField] = value.toFixed(1);
                  break;
                }
                case "fuel_used_total_return": {
                  const buffer = plainObject[key].slice(-12, -4)
                  const value = convertInvModbusHexToUint32(buffer) * 0.1
                  plainObject[baseField] = value.toFixed(1);
                  break;
                }

              }

            }
          }

          // Delete the _hex field regardless of CRC validity
          if (plainObject.packetNum == 50 || plainObject.packetNum == 56) {
            delete plainObject[key];
          }
        }
        else {
          // If the base field does not exist, delete the _hex field
          delete plainObject[key];
        }
      }
    });

    return plainObject; // Return the modified object
  });
  console.log(modifiedPackets);
  return modifiedPackets
}

// Функция для получения пакетов от сервера по id
// async function initPacketsFromServer(id) {
//   try {
//     // Находим пакеты, соответствующие переданному id
//     const packets = await Packets.find({ packetNum: Number(id) })
//       .select('-_id -__v') // исключаем поля _id и __v
//       .sort({ dt: -1 })
//       .limit(50) // ограничиваем результат 50 пакетами
//       .exec();
//     return packets;
//   } catch (error) {
//     // Логирование ошибки и переадресация ошибки вверх по цепочке
//     logger.error('Error during retrieving packets from server:', error);
//     throw error;
//   }
// }
function parseAggergationPackets(packets) {
  const packets50 = packets[0].packets50;
  const packets52 = packets[0].packets52;
  const otherPackets = packets[0].otherPackets;

  // Check if both packets50 and packets52 are present and not empty
  const bothPresent = packets50.length > 0 && packets52.length > 0;

  // Merge packets50 and packets52 if both are present
  let mergedPackets = [];
  if (bothPresent) {
    mergedPackets = packets50.map((packet, index) => {
      const packet52 = packets52[index] ? packets52[index] : {};
      delete packet52.packetNum; // Remove packetNum from packet52

      // Getting keys of packet50
      const packetKeys = Object.keys(packet);

      // Separating packet50 into two parts
      const lastTwoKeys = packetKeys.slice(-2);
      const restKeys = packetKeys.slice(0, -2);

      // Building the new object
      let newPacket = {};

      // Add the first part of packet50
      restKeys.forEach(key => {
        newPacket[key] = packet[key];
      });

      // Add packet52
      Object.assign(newPacket, packet52);

      // Add the last two fields of packet50
      lastTwoKeys.forEach(key => {
        newPacket[key] = packet[key];
      });

      return newPacket;
    });
  }
  else {
    // If only one of packets50 or packets52 is present, use that
    mergedPackets = packets50.length > 0 ? packets50 : packets52;
  }

  // Concatenate all arrays
  const finalResult = mergedPackets.concat(otherPackets);

  // finalResult is now a flat array of objects
  return finalResult;
}

async function initPacketsFromServer(payload) {
  try {
    const {
      currentPage, sortDt, packetNum, startDate, endDate, deviceId,
    } = payload;
    console.log(payload);
    const pageSize = 50; // Количество записей на странице
    const skipRecords = (currentPage - 1) * pageSize; // Количество пропускаемых записей
    const sortCriteria = sortDt ? { dt: -1 } : { pack_dt: -1 }; // Определение порядка сортировки

    // Формируем запрос на основе условий
    const query = { packetNum: Number(packetNum) };

    if (deviceId !== false) {
      query.uuid = deviceId;
    }

    if (startDate && endDate) {
      // Преобразуем строки в объекты даты Moment и затем в секунды
      const startInSeconds = moment(startDate).unix();
      const endInSeconds = moment(endDate).unix();
      query.pack_dt = { $gte: startInSeconds, $lte: endInSeconds };
    }

    // Получение общего количества записей
    const totalRecords = await Packets.countDocuments(query);

    // Получение пакетов
    // const rawPacketsO = await Packets.find(query)
    //   .select('-_id -__v') // исключаем поля _id и __v
    //   .sort(sortCriteria) // сортировка
    //   .skip(skipRecords) // пропускаем записи для пагинации
    //   .limit(pageSize) // ограничиваем количество записей
    //   .exec();
    // console.log(rawPacketsO);

    // Аггрегация пакетов
    const targetPacketNum = Number(packetNum);
    const packetNumsToInclude = targetPacketNum === 50
      ? [50, 52, targetPacketNum]
      : [targetPacketNum];

    const aggregationPipeline = [
      {
        $match: {
          packetNum: { $in: packetNumsToInclude }
        }
      },
      {
        $facet: {
          "packets50": [
            { $match: { packetNum: 50 } },

            { $sort: sortCriteria },
            { $skip: skipRecords },
            { $limit: pageSize },
            { $project: { _id: 0, __v: 0 } }
          ],
          "packets52": [
            { $match: { packetNum: 52 } },
            // Assuming no special logic for packetNum 52
            { $sort: sortCriteria },
            { $skip: skipRecords },
            { $limit: pageSize },
            { $project: { _id: 0, __v: 0 } }
          ],
          "otherPackets": [
            {
              $match: {
                $and: [
                  { packetNum: { $ne: 50 } },
                  { packetNum: { $ne: 52 } }
                ]
              }
            },
            { $sort: sortCriteria },
            { $skip: skipRecords },
            { $limit: pageSize },
            { $project: { _id: 0, __v: 0 } }
            // Your existing logic for other packets
          ]
        }
      }
    ];

    const aggroPackets = await Packets.aggregate(aggregationPipeline).exec();
    const rawPackets = parseAggergationPackets(aggroPackets);
    //console.log(rawPackets);

    // Вычисляем общее количество страниц
    const totalPages = Math.ceil(totalRecords / pageSize);


    const packets = parsePackets(rawPackets)


    return { packets, totalPages };
  } catch (error) {
    // Логирование ошибки и переадресация ошибки вверх по цепочке
    logger.error('Error during retrieving packets from server:', error);
    throw error;
  }
}



// Функция для создания пакетов на сервере из переданного payload
async function createPacketsFromServer(payload) {
  try {
    // Проверка на наличие payload и его содержимого
    if (!payload || payload.length === 0) {
      logger.error('Error while writing packet data: Payload is empty');
      return { success: false, message: 'Payload is empty' };
    }

    // Проверка на наличие uuid в первом объекте payload
    if (!payload[0].uuid) {
      logger.error('Error while writing packet data: uuid is missing');
      return { success: false, message: 'uuid is missing' };
    }

    const { uuid } = payload[0];
    // Поиск устройства по uuid
    const device = await Devices.findOne({ uuid });

    // Проверка на наличие устройства и его авторизацию
    if (!device || !device.isAuthorized) {
      logger.error(`Error while writing packet data: Device with uuid ${uuid} not found or not authorized`);
      return { success: false, message: 'Device not found or not authorized' };
    }

    // Создание массива пакетов с текущей датой и временем
    const packets = payload.map((packet) => {
      // Определяем, находится ли значение в миллисекундах или секундах
      const isMilliseconds = packet.pack_dt.toString().length > 10;

      // Преобразуем значение в секунды, если оно в миллисекундах
      const packDtInSeconds = isMilliseconds ? moment(packet.pack_dt).unix() : packet.pack_dt;

      return { ...packet, dt: new Date(), pack_dt: packDtInSeconds };
    });

    // Сохранение пакетов в базу данных
    const newPackets = await Packets.insertMany(packets);
    // запись даты последней передачи пакета
    await Devices.findOneAndUpdate(
      { uuid }, // Фильтр по uuid для поиска устройства
      { lastPacketAt: new Date() }, // Устанавливаем текущую дату и время
    );
    logger.info(`Saving packages from the device uuid ${uuid} was successful`);
    return { newPackets, success: true };
  } catch (error) {
    // Логирование ошибки и переадресация ошибки вверх по цепочке
    logger.error('Error during saving packets to server:', error);
    throw error;
  }
}

// Функция для получения пакетов от сервера на основе переданного payload
async function postInitPacketsFromServer(payload) {
  try {
    // Проверка на наличие payload
    if (!payload) {
      return { values: [], message: 'Payload is empty', code: 1 };
    }
    // Поиск устройства по параметрам из payload
    const device = await Devices.findOne({
      spsSeriesID: payload.spsSeriesID,
      spsNumber: payload.spsNumber,
      spsSection: payload.spsSection,
      litera: payload.litera,
    });
    // Проверка на наличие устройства
    if (!device) {
      return {
        values: [], message: 'Device not found', code: 1,
      };
    }

    const { uuid } = device;
    // Преобразование строковых представлений дат в объекты Date
    const beginDate = new Date(payload.dtBegin);
    const endDate = new Date(payload.dtEnd);

    // Преобразование объектов Date в секунды
    const beginSec = moment(beginDate).unix();
    const endSec = moment(endDate).unix();

    // Преобразование объектов Date в милисекунды
    const beginDateInMilliseconds = beginDate.getTime();
    const endDateInMilliseconds = endDate.getTime();

    if (!beginDate || !endDate || !beginSec || !endSec
      || !beginDateInMilliseconds || !endDateInMilliseconds) {
      return {
        values: [], message: 'Invalid date', code: 1,
      };
    }

    // Поиск пакетов, соответствующих параметрам из payload
    const packets = await Packets.find({
      // eslint-disable-next-line object-shorthand
      uuid: uuid,
      packetNum: Number(payload.packNumber),
      pack_dt: {
        $gte: beginSec,
        $lte: endSec,
      },
    });
    // Проверка на наличие пакетов
    if (packets.length === 0) {
      // Убрать после согласования данных
      // Поиск пакетов, соответствующих параметрам из payload сверка по милисекундам
      const packetsMilli = await Packets.find({
        // eslint-disable-next-line object-shorthand
        uuid: uuid,
        packetNum: Number(payload.packNumber),
        pack_dt: {
          $gte: beginDateInMilliseconds,
          $lte: endDateInMilliseconds,
        },
      });
      if (packetsMilli.length === 0) {
        console.log(packetsMilli);
        return { values: [], message: 'No packets found', code: 0 };
      }

      return { values: packetsMilli, message: 'success', code: 0 };
    }

    return { values: packets, message: 'success', code: 0 };
  } catch (error) {
    // Логирование ошибки и переадресация ошибки вверх по цепочке
    logger.error('Error during retrieving packets from server:', error);
    throw error;
  }
}

// Неиспользуемые функции
async function updatePacketsFromServer(payload) {
  return payload;
}

async function removePacketsFromServer(payload) {
  return payload;
}

// Экспорт функций для использования в других модулях
module.exports = {
  initPacketsFromServer,
  createPacketsFromServer,
  updatePacketsFromServer,
  removePacketsFromServer,
  postInitPacketsFromServer,
};
