/* eslint-disable object-shorthand */
const jwt = require('jsonwebtoken');
const Token = require('../db/models/tocken');
const logger = require('../utils/logger');

// Генерация пары токенов
const generateTokens = (payload) => {
  const accessToken = jwt.sign(payload, process.env.JWT_ACCESS_SECRET, { expiresIn: '60m' });// время жизни токена в минутах
  const refreshToken = jwt.sign(payload, process.env.JWT_REFRESH_SECRET, { expiresIn: '30d' });

  return {
    accessToken,
    refreshToken,
  };
};

// Валидация токена доступа
const isAccessTokenValid = (token) => {
  try {
    return jwt.verify(token, process.env.JWT_ACCESS_SECRET);
  } catch (error) {
    logger.error('Error during access token validation:', error.message); // Логгирование ошибки
    return null;
  }
};

// Валидация токена обновления
const isRefreshTokenValid = (token) => {
  try {
    return jwt.verify(token, process.env.JWT_REFRESH_SECRET);
  } catch (error) {
    logger.error('Error during refresh token validation:', error.message); // Логгирование ошибки
    return null;
  }
};

// Сохранение токена обновления
async function storeRefreshToken(userId, newRefreshToken) {
  try {
    // Попытка найти существующий токен обновления для данного пользователя
    let token = await Token.findOne({ userId: userId });

    if (token) {
      // Если токен найден, обновляем его значение новым токеном обновления
      token.refreshToken = newRefreshToken;
    } else {
      // Если токен не найден, создаем новую запись с токеном обновления
      token = new Token({
        userId: userId,
        refreshToken: newRefreshToken,
      });
    }
    // Сохраняем новый или обновленный токен обновления в базе данных
    await token.save();
    return token;
  } catch (error) {
    logger.error('Error during token saving:', error);
    throw error;
  }
}

// Удаление токена обновления
const deleteRefreshToken = async (refreshToken) => {
  try {
    const result = await Token.deleteOne({ refreshToken });
    return result.deletedCount > 0;
  } catch (error) {
    logger.error('Error during token delete:', error);
    return false;
  }
};

// Поиск токена обновления
const findStoredRefreshToken = async (refreshToken) => {
  try {
    const token = await Token.findOne({ refreshToken });
    return token;
  } catch (error) {
    logger.error('Error during token find:', error);
    return null;
  }
};

module.exports = {
  generateTokens,
  isAccessTokenValid,
  isRefreshTokenValid,
  storeRefreshToken,
  deleteRefreshToken,
  findStoredRefreshToken,
};
