const { loginWs, logoutWs } = require('./controllers/authorization.controller');
const { createPackets } = require('./controllers/packets.controller');
const { verifyTokenWs } = require('./middleware/verifyToken');
const { createDeviceIfNotExists } = require('./services/devices.services');
const { checkLastPacketReceived } = require('./controllers/devices.controller')
const logger = require('./utils/logger'); // Подключение логгера

function webSocet(io) {
  // middleware
  io.use((socket, next) => {
    next();
  });

  io.on('connection', (socket) => {
    logger.info('New client connected'); // Логгирование нового подключения

    const { uuid } = socket.handshake.query;
    if (uuid) {
      createDeviceIfNotExists(uuid);
    }

    verifyTokenWs(socket); // Вызов миддлвейра для проверки токена

    // Controllers
    loginWs(io, socket);
    logoutWs(io, socket);
    createPackets(io, socket);

    socket.on('disconnect', () => {
      logger.info('Client disconnected'); // Логгирование отключения клиента
    });

    // Обработчик ошибок
    socket.on('error', (error) => {
      logger.error('Socket Error:', error); // Логгирование ошибок сокета
    });
  });

  // проверка отправки пакетов от устройств
  setInterval(() => {
    checkLastPacketReceived(io);
  }, 90000);
}

module.exports = { webSocet };
